from __future__ import unicode_literals
from __future__ import absolute_import
from .responses import ErrorResponse


class AuthenticationFailed(ErrorResponse):
    pass


class NotFound(ErrorResponse):
    pass


class ServiceUnavailable(ErrorResponse):
    pass



import click
from ....cliutils import out


@click.group(help='Information on rules')
def rules():
    pass


@rules.command(help='List all rules')
@click.pass_obj
def list(state):
    out(state, state.client.rules())


@rules.command(help='Show details about a specific rule')
@click.argument('id',
                type=int)
@click.pass_obj
def show(state, id):
    out(state, state.client.rules(id))



from __future__ import unicode_literals
from __future__ import absolute_import
from .util import must_get, camelize


class Model(object):
    def __init__(self, data):
        self.data = data

    def __getattr__(self, name):
        candidate_key = camelize(name, True)

        if candidate_key in self.data:
            return must_get(self.data, candidate_key)
        else:
            return super(self, name)

    def to_dict(self):
        return self.data



from __future__ import unicode_literals
from __future__ import absolute_import
import os
from .client import Client


def client():
    # build a test client, falling back to the well-known test API key
    c = Client(os.environ.get('PERFORMLINE_API_KEY', '00e4db7592541054d3791f42d62524f4139705c4'),
               loglevel=os.environ.get('LOGLEVEL', 'WARNING'))

    url = os.environ.get('PERFORMLINE_API_URL')
    prefix = os.environ.get('PERFORMLINE_API_PREFIX')

    if url is not None:
        c.url = url

    if prefix is not None:
        c.prefix = prefix

    return c



import click
from ....cliutils import out


@click.group(help='Information on campaigns')
def campaigns():
    pass


@campaigns.command(help='List all campaigns')
@click.pass_obj
def list(state):
    out(state, state.client.campaigns())


@campaigns.command(help='Show details about a campaign')
@click.argument('id',
                type=int)
@click.pass_obj
def show(state, id):
    out(state, state.client.campaigns(id))



from __future__ import absolute_import
from .models import Brand, Campaign, Rule, TrafficSource


class CommonClientMethods(object):
    def brands(self, id=None):
        if id is None:
            response = self.request('get', '/common/brands/')
            return self.wrap_response(response, Brand)
        else:
            response = self.request('get', '/common/brands/%d/' % id)
            return self.wrap_response(response, Brand, flat=True)

    def campaigns(self, id=None):
        if id is None:
            response = self.request('get', '/common/campaigns/')
            return self.wrap_response(response, Campaign)
        else:
            response = self.request('get', '/common/campaigns/%d/' % id)
            return self.wrap_response(response, Campaign, flat=True)

    def rules(self, id=None):
        if id is None:
            response = self.request('get', '/common/rules/')
            return self.wrap_response(response, Rule)
        else:
            response = self.request('get', '/common/rules/%d/' % id)
            return self.wrap_response(response, Rule, flat=True)

    def trafficsources(self, id=None):
        if id is None:
            response = self.request('get', '/common/trafficsources/')
            return self.wrap_response(response, TrafficSource)
        else:
            response = self.request('get', '/common/trafficsources/%d/' % id)
            return self.wrap_response(response, TrafficSource, flat=True)



from __future__ import unicode_literals
from __future__ import absolute_import
import re

CAMEL_OMIT = r'[\W\s\_\-]+'
CAMEL_SPLIT = re.compile(r'(?:([A-Z][^A-Z]*)|'+CAMEL_OMIT+')')
CAMEL_STRIP = re.compile(CAMEL_OMIT)
DATA_ELEMENT_SEPARATOR = '/'


class MissingKeyException(Exception):
    pass


def must_get(data, path):
    if not isinstance(data, dict):
        raise Exception("Response data is unavailable")

    current = data
    parts = path.split(DATA_ELEMENT_SEPARATOR)

    for i in range(0, len(parts)):
        if isinstance(current, dict) and parts[i] in current:
            current = current[parts[i]]
        else:
            raise MissingKeyException("Cannot retrieve data element '%s'", path)

    return current


def get(data, path, fallback=None):
    try:
        return must_get(data, path)
    except MissingKeyException:
        return fallback


def camelize(value, upperFirst=False):
    # split on word-separating characters (and discard them), or on capital
    # letters (preserving them)
    value = CAMEL_SPLIT.split(value)

    # filter out None and empty/only-whitespace strings from the list
    value = [x for x in value if x is not None and not x.strip() == '']

    # capitalize each word
    value = ''.join(x.title() for x in value)

    # do a final pass removing omitted characters from the value
    value = re.sub(CAMEL_STRIP, '', value)

    if upperFirst:
        # return PascalCase
        return value
    else:
        # return camelCase
        return value[0].lower() + value[1:]



from __future__ import unicode_literals
from __future__ import absolute_import
import requests
import logging
import datetime
from .models import Model
from .responses import SuccessResponse, ErrorResponse
from .exceptions import AuthenticationFailed, \
                        NotFound, \
                        ServiceUnavailable
from .products.common.api import CommonClientMethods

ALLOWED_METHODS = ['get', 'post', 'put', 'delete', 'options', 'head']
RX_TRIM_SLASH = r'(?:^/*|/*$)'


class Client(CommonClientMethods, object):
    url = 'https://api.performmatch.com'
    prefix = '/'

    def __init__(self, token, loglevel='WARNING'):
        self.token = token

        if isinstance(loglevel, str):
            loglevel = loglevel.upper()

        lvl = logging.getLevelName(loglevel)

        if isinstance(lvl, int):
            self.loglevel = lvl
        else:
            self.loglevel = logging.INFO

        logging.basicConfig(level=self.loglevel)

    def request(self, method, path, data=None, params={}, headers={}):
        if not method.lower() in ALLOWED_METHODS:
            raise Exception('Invalid request method %s' % method)

        # add token to Authorization header
        headers['Authorization'] = 'Token %s' % self.token

        response = getattr(requests, method)(
            '%s/%s%s/' % (
                self.url.strip('/'),
                self.prefix.lstrip('/'),
                path.strip('/'),
            ),
            data=data,
            params=params,
            headers=headers)

        if response.headers.get('X-PerformLine-Deprecated') == '1':
            _notAfter = response.headers.get('X-PerformLine-Deprecated-After')
            afterMsg = ''

            if _notAfter is not None:
                notAfter = datetime.datetime.strptime(_notAfter, '%Y-%m-%dT%H:%M:%S.%f')
                afterMsg = ' It will stop working after %s. ' % notAfter

            logging.warning('The API endpoint "%s" has been marked deprecated.%s' % afterMsg)
            logging.warning((
                'Upgrade the performline Python module to the latest version to '
                'stop seeing this message.'
            ))

        if response.status_code < 400:
            return SuccessResponse(response.json())
        else:
            if response.status_code == 403:
                raise AuthenticationFailed(response.json())
            elif response.status_code == 404:
                raise NotFound(response.json())
            elif response.status_code >= 500:
                raise ServiceUnavailable(response.json())
            else:
                raise ErrorResponse(response.json())

    def wrap_response(self, response, model, flat=False):
        if isinstance(response, SuccessResponse) and response.length() > 0:
            if issubclass(model, Model):
                models = [model(i) for i in response.results()]

                if flat and len(models) == 1:
                    return models[0]
                else:
                    return models

        return None



from __future__ import absolute_import
import click
from ....cliutils import out


@click.group(help='Brands associated with your account')
def brands():
    pass


@brands.command(help='List all brands')
@click.pass_obj
def list(state):
    out(state, state.client.brands())


@brands.command(help='Show details about a single brand')
@click.argument('id',
                type=int)
@click.pass_obj
def show(state, id):
    out(state, state.client.brands(id))



from __future__ import absolute_import
import click
import sys
import json
import yaml


def normalize(data):
    if hasattr(data, 'to_dict'):
        data = data.to_dict()
    elif isinstance(data, list):
        if len(data) > 0:
            data = [normalize(i) for i in data]

    return data


def errout(message, exitcode=1):
    sys.stderr.write('%s\n' % message)
    exit(exitcode)


def out(state, data=None):
    rv = ''
    data = normalize(data)

    if state.fmt == 'json':
        rv = json.dumps(data, indent=4)
    elif state.fmt == 'yaml':
        rv = yaml.safe_dump(data, default_flow_style=False)
    else:
        rv = str(data)

    if len(rv) > 0:
        click.echo(rv)



from __future__ import unicode_literals
from __future__ import absolute_import
from .util import get, must_get


class Response(object):
    def __init__(self, data={}):
        self.data = data

    def status(self):
        return self.must_get('Status')

    def must_get(self, path):
        return must_get(self.data, path)

    def get(self, path, fallback=None):
        return get(self.data, path, fallback)


class SuccessResponse(Response):
    def total_length(self):
        return self.get('ResultCount/Total', 0)

    def length(self):
        return self.get('ResultCount/Current', self.total_length())

    def page_count(self):
        return self.get('ResultCount/Pages', 0)

    def results(self):
        return self.get('Results', [])


class ErrorResponse(Response, Exception):
    def __init__(self, data={}):
        self.data = data
        self.message = self.get('ErrorDetails', self.get('ErrorMessage'))

        super(Exception, self).__init__(self.message)



from __future__ import absolute_import
from .client import Client
from .responses import ErrorResponse
from .cliutils import errout
from .products.common.cli.brands import brands
from .products.common.cli.campaigns import campaigns
from .products.common.cli.rules import rules
from .products.common.cli.trafficsources import sources
import click


class State(object):
    def __init__(self, context, options={}):
        self.context = context
        self.token = options.get('token')
        self.url = options.get('url')
        self.prefix = options.get('prefix')
        self.loglevel = options.get('loglevel')
        self.fmt = options.get('format')

        if self.loglevel is not None:
            self.client = Client(self.token, loglevel=self.loglevel)
        else:
            self.client = Client(self.token)

        if self.url:
            self.client.url = self.url

        if self.prefix:
            self.client.prefix = self.prefix


@click.group()
@click.option('--api-key', '-k',
              metavar='KEY',
              help=(
                'The API key to use for authentication '
                '[$PERFORMLINE_API_KEY]'
              ),
              required=True,
              envvar='API_KEY')
@click.option('--api-host',
              metavar='URL',
              help=(
                  'The URL to use to connect to the PerformMatch API service '
                  '(this should normally not need to be changed)\n'
                  '[$PERFORMLINE_API_URL]'
              ),
              envvar='API_URL')
@click.option('--api-endpoint-prefix',
              metavar='PATH',
              help=(
                  'A string to prefix API endpoint paths with '
                  '(this should normally not need to be changed)\n'
                  '[$PERFORMLINE_API_PREFIX]'
              ),
              envvar='API_PREFIX')
@click.option('--log-level', '-L',
              default='warning',
              help=(
                  'Set the level of logging verbosity\n'
                  '[$PERFORMLINE_LOGLEVEL]'
              ),
              type=click.Choice([
                'debug', 'info', 'warning', 'error',
              ]),
              envvar='LOGLEVEL')
@click.option('--format', '-f',
              default='yaml',
              type=click.Choice([
                'yaml', 'json',
              ]))
@click.pass_context
def main(ctx,
         api_key,
         api_host,
         api_endpoint_prefix,
         log_level,
         format):
    """PerformMatch Compliance API client utility"""
    ctx.obj = State(ctx, {
        'token': api_key,
        'url': api_host,
        'prefix': api_endpoint_prefix,
        'loglevel': log_level,
        'format': format,
    })

main.add_command(brands)
main.add_command(campaigns)
main.add_command(rules)
main.add_command(sources)

try:
    main(auto_envvar_prefix='PERFORMLINE')
except Exception as e:
    if isinstance(e, ErrorResponse):
        errout('The PerformMatch API encountered an error: %s' % e.message)
    else:
        errout('Unhandled Error %s: %s' % (e.__class__.__name__, e.message))



from setuptools import setup, find_packages

setup(
    name='performline',
    description='Client library for using the public PerformMatch Compliance API',
    version='0.1.0a6',
    author='PerformLine Engineering',
    author_email='tech+api@performline.com',
    url='https://github.com/PerformLine/python-performline-client',
    install_requires=[
        'click==6.6',
        'requests==2.11.1',
        'pyyaml',
    ],
    entry_points={
        'console_scripts': [
            'performline=performline.cli:main',
        ],
    },
    packages=find_packages(exclude=['*.tests']),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Office/Business',
        'Environment :: Console',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows',
        'Intended Audience :: Customer Service',
        'Intended Audience :: Financial and Insurance Industry',
    ],
)



from __future__ import unicode_literals
from __future__ import absolute_import
from ...models import Model


class Brand(Model):
    pass


class Campaign(Model):
    pass


class Rule(Model):
    pass


class TrafficSource(Model):
    pass



import click
from ....cliutils import out


@click.group(help='Information on traffic sources')
def sources():
    pass


@sources.command(help='List all traffic sources')
@click.pass_obj
def list(state):
    out(state, state.client.trafficsources())


@sources.command(help='Show details about a traffic source')
@click.argument('id',
                type=int)
@click.pass_obj
def show(state, id):
    out(state, state.client.trafficsources(id))
